// Copyright 2018 Espressif Systems (Shanghai) PTE LTD
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef _UTIME_H_
#define _UTIME_H_

#include <sys/time.h>

#ifdef __cplusplus
extern "C" {
#endif

struct utimbuf {
    time_t actime;       // access time
    time_t modtime;      // modification time
};

int utime(const char *path, const struct utimbuf *times);

#ifdef __cplusplus
};
#endif

#endif /* _UTIME_H_ */
