/*
 * SPDX-FileCopyrightText: 2021 Espressif Systems (Shanghai) CO LTD
 *
 * SPDX-License-Identifier: Apache-2.0
 */
#pragma once

#ifdef __cplusplus
extern "C" {
#endif

#define TRACEMEM_BLK0_ADDR                      0x3FFFC000UL
#define TRACEMEM_BLK1_ADDR                      0x3FFF8000UL

#ifdef __cplusplus
}
#endif
