// Copyright 2020 Espressif Systems (Shanghai) PTE LTD
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at

//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef _SOC_ADC_CHANNEL_H
#define _SOC_ADC_CHANNEL_H

#define ADC1_GPIO1_CHANNEL     ADC1_CHANNEL_0
#define ADC1_CHANNEL_0_GPIO_NUM 0

#define ADC1_GPIO2_CHANNEL     ADC1_CHANNEL_1
#define ADC1_CHANNEL_1_GPIO_NUM 1

#define ADC1_GPIO3_CHANNEL     ADC1_CHANNEL_2
#define ADC1_CHANNEL_2_GPIO_NUM 2

#define ADC1_GPIO4_CHANNEL     ADC1_CHANNEL_3
#define ADC1_CHANNEL_3_GPIO_NUM 3

#define ADC1_GPIO5_CHANNEL     ADC1_CHANNEL_4
#define ADC1_CHANNEL_4_GPIO_NUM 4

#define ADC2_GPIO5_CHANNEL      ADC2_CHANNEL_0
#define ADC2_CHANNEL_0_GPIO_NUM 5

#endif
