// Copyright 2010-2016 Espressif Systems (Shanghai) PTE LTD
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at

//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef _SOC_DAC_CHANNEL_H
#define _SOC_DAC_CHANNEL_H

#define DAC_GPIO17_CHANNEL      DAC_CHANNEL_1
#define DAC_CHANNEL_1_GPIO_NUM  17

#define DAC_GPIO18_CHANNEL      DAC_CHANNEL_2
#define DAC_CHANNEL_2_GPIO_NUM  18

#endif
